@@ -1,11 +1,12 @@
 """Private logic for creating pydantic dataclasses."""
+
 from __future__ import annotations as _annotations
 
 import dataclasses
 import typing
 import warnings
 from functools import partial, wraps
-from typing import Any, Callable, ClassVar
+from typing import Any, ClassVar
 
 from pydantic_core import (
     ArgsKwargs,
@@ -16,27 +17,23 @@ from pydantic_core import (
 from typing_extensions import TypeGuard
 
 from ..errors import PydanticUndefinedAnnotation
-from ..fields import FieldInfo
-from ..plugin._schema_validator import create_schema_validator
+from ..plugin._schema_validator import PluggableSchemaValidator, create_schema_validator
 from ..warnings import PydanticDeprecatedSince20
-from . import _config, _decorators, _typing_extra
+from . import _config, _decorators
 from ._fields import collect_dataclass_fields
 from ._generate_schema import GenerateSchema
 from ._generics import get_standard_typevars_map
 from ._mock_val_ser import set_dataclass_mocks
+from ._namespace_utils import NsResolver
 from ._schema_generation_shared import CallbackGetCoreSchemaHandler
 from ._signature import generate_pydantic_signature
+from ._utils import LazyClassAttribute
 
 if typing.TYPE_CHECKING:
+    from _typeshed import DataclassInstance as StandardDataclass
+
     from ..config import ConfigDict
-
-    class StandardDataclass(typing.Protocol):
-        __dataclass_fields__: ClassVar[dict[str, Any]]
-        __dataclass_params__: ClassVar[Any]  # in reality `dataclasses._DataclassParams`
-        __post_init__: ClassVar[Callable[..., None]]
-
-        def __init__(self, *args: object, **kwargs: object) -> None:
-            pass
+    from ..fields import FieldInfo
 
     class PydanticDataclass(StandardDataclass, typing.Protocol):
         """A protocol containing attributes only available once a class has been decorated as a Pydantic dataclass.
@@ -57,7 +54,7 @@ if typing.TYPE_CHECKING:
         __pydantic_decorators__: ClassVar[_decorators.DecoratorInfos]
         __pydantic_fields__: ClassVar[dict[str, FieldInfo]]
         __pydantic_serializer__: ClassVar[SchemaSerializer]
-        __pydantic_validator__: ClassVar[SchemaValidator]
+        __pydantic_validator__: ClassVar[SchemaValidator | PluggableSchemaValidator]
 
 else:
     # See PyCharm issues https://youtrack.jetbrains.com/issue/PY-21915
@@ -67,18 +64,20 @@ else:
 
 def set_dataclass_fields(
     cls: type[StandardDataclass],
-    types_namespace: dict[str, Any] | None = None,
+    ns_resolver: NsResolver | None = None,
     config_wrapper: _config.ConfigWrapper | None = None,
 ) -> None:
     """Collect and set `cls.__pydantic_fields__`.
 
     Args:
         cls: The class.
-        types_namespace: The types namespace, defaults to `None`.
+        ns_resolver: Namespace resolver to use when getting dataclass annotations.
         config_wrapper: The config wrapper instance, defaults to `None`.
     """
     typevars_map = get_standard_typevars_map(cls)
-    fields = collect_dataclass_fields(cls, types_namespace, typevars_map=typevars_map, config_wrapper=config_wrapper)
+    fields = collect_dataclass_fields(
+        cls, ns_resolver=ns_resolver, typevars_map=typevars_map, config_wrapper=config_wrapper
+    )
 
     cls.__pydantic_fields__ = fields  # type: ignore
 
@@ -88,7 +87,8 @@ def complete_dataclass(
     config_wrapper: _config.ConfigWrapper,
     *,
     raise_errors: bool = True,
-    types_namespace: dict[str, Any] | None,
+    ns_resolver: NsResolver | None = None,
+    _force_build: bool = False,
 ) -> bool:
     """Finish building a pydantic dataclass.
 
@@ -100,7 +100,10 @@ def complete_dataclass(
         cls: The class.
         config_wrapper: The config wrapper instance.
         raise_errors: Whether to raise errors, defaults to `True`.
-        types_namespace: The types namespace.
+        ns_resolver: The namespace resolver instance to use when collecting dataclass fields
+            and during schema building.
+        _force_build: Whether to force building the dataclass, no matter if
+            [`defer_build`][pydantic.config.ConfigDict.defer_build] is set.
 
     Returns:
         `True` if building a pydantic dataclass is successfully completed, `False` otherwise.
@@ -108,32 +111,10 @@ def complete_dataclass(
     Raises:
         PydanticUndefinedAnnotation: If `raise_error` is `True` and there is an undefined annotations.
     """
-    if hasattr(cls, '__post_init_post_parse__'):
-        warnings.warn(
-            'Support for `__post_init_post_parse__` has been dropped, the method will not be called', DeprecationWarning
-        )
+    original_init = cls.__init__
 
-    if types_namespace is None:
-        types_namespace = _typing_extra.get_cls_types_namespace(cls)
-
-    set_dataclass_fields(cls, types_namespace, config_wrapper=config_wrapper)
-
-    typevars_map = get_standard_typevars_map(cls)
-    gen_schema = GenerateSchema(
-        config_wrapper,
-        types_namespace,
-        typevars_map,
-    )
-
-    # This needs to be called before we change the __init__
-    sig = generate_pydantic_signature(
-        init=cls.__init__,
-        fields=cls.__pydantic_fields__,  # type: ignore
-        config_wrapper=config_wrapper,
-        is_dataclass=True,
-    )
-
-    # dataclass.__init__ must be defined here so its `__qualname__` can be changed since functions can't be copied.
+    # dataclass.__init__ must be defined here so its `__qualname__` can be changed since functions can't be copied,
+    # and so that the mock validator is used if building was deferred:
     def __init__(__dataclass_self__: PydanticDataclass, *args: Any, **kwargs: Any) -> None:
         __tracebackhide__ = True
         s = __dataclass_self__
@@ -143,7 +124,40 @@ def complete_dataclass(
 
     cls.__init__ = __init__  # type: ignore
     cls.__pydantic_config__ = config_wrapper.config_dict  # type: ignore
-    cls.__signature__ = sig  # type: ignore
+
+    if not _force_build and config_wrapper.defer_build:
+        set_dataclass_mocks(cls, cls.__name__)
+        return False
+
+    if hasattr(cls, '__post_init_post_parse__'):
+        warnings.warn(
+            'Support for `__post_init_post_parse__` has been dropped, the method will not be called', DeprecationWarning
+        )
+
+    set_dataclass_fields(cls, ns_resolver, config_wrapper=config_wrapper)
+
+    typevars_map = get_standard_typevars_map(cls)
+    gen_schema = GenerateSchema(
+        config_wrapper,
+        ns_resolver=ns_resolver,
+        typevars_map=typevars_map,
+    )
+
+    # set __signature__ attr only for model class, but not for its instances
+    # (because instances can define `__call__`, and `inspect.signature` shouldn't
+    # use the `__signature__` attribute and instead generate from `__call__`).
+    cls.__signature__ = LazyClassAttribute(
+        '__signature__',
+        partial(
+            generate_pydantic_signature,
+            # It's' important that we reference the original_init here
+            init=original_init,
+            fields=cls.__pydantic_fields__,  # type: ignore
+            populate_by_name=config_wrapper.populate_by_name,
+            extra=config_wrapper.extra,
+            is_dataclass=True,
+        ),
+    )
     get_core_schema = getattr(cls, '__get_pydantic_core_schema__', None)
     try:
         if get_core_schema:
@@ -163,7 +177,7 @@ def complete_dataclass(
         set_dataclass_mocks(cls, cls.__name__, f'`{e.name}`')
         return False
 
-    core_config = config_wrapper.core_config(cls)
+    core_config = config_wrapper.core_config(title=cls.__name__)
 
     try:
         schema = gen_schema.clean_schema(schema)
@@ -190,6 +204,7 @@ def complete_dataclass(
 
         cls.__setattr__ = validated_setattr.__get__(None, cls)  # type: ignore
 
+    cls.__pydantic_complete__ = True
     return True
 
 
@@ -201,7 +216,7 @@ def is_builtin_dataclass(_cls: type[Any]) -> TypeGuard[type[StandardDataclass]]:
     - `_cls` does not inherit from a processed pydantic dataclass (and thus have a `__pydantic_validator__`)
     - `_cls` does not have any annotations that are not dataclass fields
     e.g.
-    ```py
+    ```python
     import dataclasses
 
     import pydantic.dataclasses
